package com.ads.business.controller;


import com.ads.business.entity.HsayiCategoryInfo;
import com.ads.business.model.rep.CategoryInfoAddRepVo;
import com.ads.business.model.rep.CategoryInfoAuthedEditRepVo;
import com.ads.business.model.rep.CategoryInfoEditRepVo;
import com.ads.business.model.rep.CategoryInfoReqVo;
import com.ads.business.model.resq.CategoryInfoDetailRespVo;
import com.ads.business.model.resq.CategoryInfoRespVo;
import com.ads.business.model.resq.CategoryListRespVo;
import com.ads.business.service.HsayiCategoryInfoService;
import com.ads.common.base.BaseController;
import com.ads.common.data.JsonResult;
import com.ads.common.exception.BusinessException;
import com.ads.common.util.UtilConst;
import com.ads.common.util.UtilMessage;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 类目表 前端控制器
 * </p>
 *
 * @author huzhihao
 * @since 2021-12-24
 */
@RestController
@RequestMapping("/api/category")
@Api(tags = "类目控制器")
@ApiIgnore
public class HsayiCategoryInfoController extends BaseController {


    @Resource
    private HsayiCategoryInfoService categoryInfoService;

    /**
     * 类目分页列表
     *
     * @param vo 入参
     * @return 返回结果对象
     */
    @GetMapping("/page")
    @ApiOperation(value = "类目分页列表", notes = "类目分页列表", response = CategoryInfoRespVo.class)
    public JsonResult page(CategoryInfoReqVo vo) {
        return categoryInfoService.findPage(vo);
    }

    /**
     * 类目详情
     *
     * @param categoryInfoId 主键ID
     * @return 返回结果对象
     */
    @ApiOperation(value = "类目详情", notes = "类目详情", response = CategoryInfoDetailRespVo.class)
    @GetMapping("/detail")
    public JsonResult detail(@RequestParam(value = "categoryInfoId") Long categoryInfoId) {
        return JsonResult.success(categoryInfoService.detail(categoryInfoId), UtilMessage.GET_MESSAGE_SUCCESS);
    }

    /**
     * 新增
     *
     * @param vo 入参对象
     * @return 返回结果对象
     */
    @ApiOperation(value = "新增", notes = "新增")
    @PostMapping("/add")
    public JsonResult add(@Valid @RequestBody CategoryInfoAddRepVo vo) {
        return categoryInfoService.add(vo);
    }


    /**
     * 修改
     *
     * @param vo 入参对象
     * @return 返回结果对象
     */
    @ApiOperation(value = "修改", notes = "修改")
    @PutMapping("/edit")
    public JsonResult edit(@Valid @RequestBody CategoryInfoEditRepVo vo) {
        return categoryInfoService.edit(vo);
    }
    /**
     * 审核通过的修改
     *
     * @param vo 入参对象
     * @return 返回结果对象
     */
    @ApiOperation(value = "审核通过的修改", notes = "审核通过的修改")
    @PutMapping("/authedEdit")
    public JsonResult authedEdit(@Valid @RequestBody CategoryInfoAuthedEditRepVo vo) {
        return categoryInfoService.authedEdit(vo);
    }

    /**
     * 删除
     */
    @DeleteMapping("/{categoryInfoIds}")
    @ApiOperation(value = "删除", notes = "删除")
    public JsonResult remove(@PathVariable Long[] categoryInfoIds) {
        if (categoryInfoIds.length <= 0) {
            throw new BusinessException("请选择要删除的数据");
        }
        return categoryInfoService.deleteByIds(categoryInfoIds);
    }

    /**
     * 获取审核通过的类目
     *
     * @return 返回结果对象
     */
    @ApiOperation(value = "获取审核通过的类目", notes = "获取审核通过的类目", response = CategoryListRespVo.class)
    @GetMapping("/getCategoryList")
    public JsonResult getCategoryList() {
        LambdaQueryWrapper<HsayiCategoryInfo> wrapper = new LambdaQueryWrapper<>();
        wrapper.select(HsayiCategoryInfo::getLevel3, HsayiCategoryInfo::getLevel3_name);
        wrapper.eq(HsayiCategoryInfo::getStatus, UtilConst.authStatus);
        List<HsayiCategoryInfo> list = categoryInfoService.list(wrapper);
        List<CategoryListRespVo> results = Lists.newArrayList();
        list.stream().forEach(t -> {
            CategoryListRespVo respVo = new CategoryListRespVo();
            respVo.setLevel3(t.getLevel3());
            respVo.setLevel3_name(t.getLevel3_name());
            results.add(respVo);
        });
        return JsonResult.success(results, UtilMessage.GET_MESSAGE_SUCCESS);
    }
}
