package com.ads.business.controller;


import com.ads.business.model.rep.CommodityAddRepVo;
import com.ads.business.model.rep.CommodityEditRepVo;
import com.ads.business.model.rep.CommodityReqVo;
import com.ads.business.model.resq.CommodityDetailVo;
import com.ads.business.model.resq.CommodityRespVo;
import com.ads.business.service.HsayiCommodityService;
import com.ads.common.base.BaseController;
import com.ads.common.data.JsonResult;
import com.ads.common.exception.BusinessException;
import com.ads.common.util.UtilMessage;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.annotation.Resource;
import javax.validation.Valid;

/**
 * <p>
 * 商品表 前端控制器
 * </p>
 *
 * @author huzhihao
 * @since 2021-12-24
 */
@RestController
@RequestMapping("/api/commodity")
@Api(tags = "商品控制器")
@ApiIgnore
public class HsayiCommodityController extends BaseController {

    @Resource
    private HsayiCommodityService commodityService;


    /**
     * 商品分页列表统计各类型的数量
     *
     * @return 返回结果对象
     */
    @GetMapping("/getCountByType")
    @ApiOperation(value = "商品分页列表统计各类型的数量", notes = "商品分页列表统计各类型的数量")
    public JsonResult getCountByType() {
        return JsonResult.success(commodityService.getCountByType(), UtilMessage.GET_MESSAGE_SUCCESS);
    }

    /**
     * 商品分页列表
     *
     * @param vo 入参
     * @return 返回结果对象
     */
    @GetMapping("/page")
    @ApiOperation(value = "商品分页列表", notes = "商品分页列表", response = CommodityRespVo.class)
    public JsonResult page(CommodityReqVo vo) {
        return commodityService.findPage(vo);
    }


    /**
     * 商品详情
     *
     * @param outProductId 主键ID
     * @return 返回结果对象
     */
    @ApiOperation(value = "商品详情", notes = "商品详情", response = CommodityDetailVo.class)
    @GetMapping("/detail")
    public JsonResult detail(@RequestParam(value = "outProductId") String outProductId) {
        return JsonResult.success(commodityService.detail(outProductId), UtilMessage.GET_MESSAGE_SUCCESS);
    }

    /**
     * 商品提交新审核的编辑详情
     *
     * @param outProductId 主键ID
     * @return 返回结果对象
     */
    @ApiOperation(value = "商品提交新审核的编辑详情", notes = "商品提交新审核的编辑详情", response = CommodityDetailVo.class)
    @GetMapping("/editDetail")
    public JsonResult editDetail(@RequestParam(value = "outProductId") String outProductId) {
        return JsonResult.success(commodityService.editDetail(outProductId), UtilMessage.GET_MESSAGE_SUCCESS);
    }
    /**
     * 商品审核中详情
     *
     * @param outProductId 主键ID
     * @return 返回结果对象
     */
    @ApiOperation(value = "商品审核中详情", notes = "商品审核中详情", response = CommodityDetailVo.class)
    @GetMapping("/authDetail")
    public JsonResult authDetail(@RequestParam(value = "outProductId") String outProductId) {
        return JsonResult.success(commodityService.authDetail(outProductId), UtilMessage.GET_MESSAGE_SUCCESS);
    }

    /**
     * 新增
     *
     * @param vo 入参对象
     * @return 返回结果对象
     */
    @ApiOperation(value = "新增", notes = "新增")
    @PostMapping("/add")
    public JsonResult add(@Valid @RequestBody CommodityAddRepVo vo) {
        return commodityService.add(vo);
    }


    /**
     * 修改
     *
     * @param vo 入参对象
     * @return 返回结果对象
     */
    @ApiOperation(value = "修改", notes = "修改")
    @PutMapping("/edit")
    public JsonResult edit(@Valid @RequestBody CommodityEditRepVo vo) {
        return commodityService.edit(vo);
    }

    /**
     * 删除
     */
    @DeleteMapping("/{outProductIds}")
    @ApiOperation(value = "删除", notes = "删除")
    public JsonResult remove(@PathVariable String[] outProductIds) {
        if (outProductIds.length <= 0) {
            throw new BusinessException("请选择要删除的数据");
        }
        return commodityService.deleteByIds(outProductIds);
    }

    /**
     * 上下架
     */
    @GetMapping("/changeStatus")
    @ApiOperation(value = "上下架", notes = "上下架")
    public JsonResult changeEditStatus(@RequestParam(value = "commodityId") Long commodityId, @RequestParam(value = "editStatus") Integer editStatus) {
        return commodityService.changeEditStatus(commodityId, editStatus);
    }

}
