package com.ads.business.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import com.ads.business.dao.HsayiAftersaleInfoDao;
import com.ads.business.entity.HsayiAftersaleInfo;
import com.ads.business.entity.HsayiAftersaleProduct;
import com.ads.business.entity.HsayiOrderInfo;
import com.ads.business.model.rep.AftersaleAddReqVo;
import com.ads.business.service.HsayiAftersaleInfoService;
import com.ads.business.service.HsayiAftersaleProductService;
import com.ads.business.service.HsayiOrderInfoService;
import com.ads.business.service.WechatFacade;
import com.ads.common.exception.BusinessException;
import com.ads.common.util.SecurityUtil;
import com.ads.common.util.UtilMessage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 售后信息表 服务实现类
 * </p>
 *
 * @author huzhihao
 * @since 2022-01-06
 */
@Service
public class HsayiAftersaleInfoServiceImpl extends ServiceImpl<HsayiAftersaleInfoDao, HsayiAftersaleInfo> implements HsayiAftersaleInfoService {

    @Autowired
    private HsayiAftersaleProductService aftersaleProductService;

    @Autowired
    private WechatFacade wechatFacade;

    @Autowired
    private HsayiOrderInfoService orderInfoService;


    /**
     * 同步售后  根据outAftersaleId重入
     *
     * @param vo
     * @return void
     * @author huzhihao
     * @date 2022/1/6 17:31
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void syncAftersale(AftersaleAddReqVo vo) {
        //查询订单是否存在
        List<HsayiOrderInfo> orderList = orderInfoService.list(Wrappers.<HsayiOrderInfo>lambdaQuery()
                .eq(HsayiOrderInfo::getOut_order_id, vo.getOut_order_id()));
        if (CollUtil.isEmpty(orderList)) {
            throw new BusinessException(UtilMessage.ORDER_NOT_FOUND);
        }
        String outAftersaleId = vo.getOut_aftersale_id();
        //装配售后实体
        HsayiAftersaleInfo entity = new HsayiAftersaleInfo();
        BeanUtil.copyProperties(vo, entity);
        entity.setCreate_time(DateUtil.parse(vo.getCreate_time(), DatePattern.NORM_DATETIME_PATTERN));
        //装配售后商品实体
        List<HsayiAftersaleProduct> aftersaleProductList = vo.getProduct_infos().stream().map(t -> {
            HsayiAftersaleProduct aftersaleProduct = new HsayiAftersaleProduct();
            BeanUtil.copyProperties(t, aftersaleProduct);
            aftersaleProduct.setOut_aftersale_id(outAftersaleId);
            return aftersaleProduct;
        }).collect(Collectors.toList());
        //判断outAftersaleId数据是否已存在
        HsayiAftersaleInfo old = this.getOne(Wrappers.<HsayiAftersaleInfo>lambdaQuery()
                .eq(HsayiAftersaleInfo::getOut_aftersale_id, outAftersaleId));
        if (!Objects.isNull(old)) {
            //存在则更新原始数据
            entity.setAftersale_id(old.getAftersale_id());
        }
        //保存售后信息
        this.saveOrUpdate(entity);
        //保存售后商品信息
        aftersaleProductService.deleteSoftlyByOutAftersaleId(outAftersaleId);
        aftersaleProductService.saveOrUpdateBatch(aftersaleProductList);
        //调用微信创建售后
        wechatFacade.aftersaleAdd(vo);
    }

    /**
     * 同步售后  根据outAftersaleId重入
     *
     * @param data
     * @return void
     * @author huzhihao
     * @date 2022/1/6 17:31
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public void syncAftersale(String data) {
        AftersaleAddReqVo aftersaleAddReqVo;
        try {
            aftersaleAddReqVo = (AftersaleAddReqVo) SecurityUtil.decrypt(data, AftersaleAddReqVo.class);
        } catch (Exception e) {
            throw new BusinessException(UtilMessage.VALIDATE_ERROR);
        }
        this.syncAftersale(aftersaleAddReqVo);
    }
}
