package com.ads.business.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.ads.business.dao.HsayiCategoryInfoDao;
import com.ads.business.entity.HsayiCategoryImg;
import com.ads.business.entity.HsayiCategoryInfo;
import com.ads.business.model.rep.CategoryInfoAddRepVo;
import com.ads.business.model.rep.CategoryInfoAuthedEditRepVo;
import com.ads.business.model.rep.CategoryInfoEditRepVo;
import com.ads.business.model.rep.CategoryInfoReqVo;
import com.ads.business.model.resq.CategoryInfoDetailRespVo;
import com.ads.business.model.resq.CategoryInfoRespVo;
import com.ads.business.service.HsayiCategoryImgService;
import com.ads.business.service.HsayiCategoryInfoService;
import com.ads.business.service.WechatFacade;
import com.ads.common.data.JsonResult;
import com.ads.common.exception.BusinessException;
import com.ads.common.util.PageUtils;
import com.ads.common.util.UtilConst;
import com.ads.common.util.UtilMessage;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 类目表 服务实现类
 * </p>
 *
 * @author huzhihao
 * @since 2021-12-24
 */
@Service
public class HsayiCategoryInfoServiceImpl extends ServiceImpl<HsayiCategoryInfoDao, HsayiCategoryInfo> implements HsayiCategoryInfoService {


    @Resource
    private HsayiCategoryImgService categoryImgService;

    @Resource
    private WechatFacade wechatFacade;

    @Override
    public JsonResult findPage(CategoryInfoReqVo vo) {
        Long currPage = vo.getPage() == null ? 1 : vo.getPage().longValue();
        Long pageSize = vo.getPageSize() == null ? 10 : vo.getPageSize().longValue();
        Page<CategoryInfoRespVo> page = new Page<>(currPage, pageSize);
        Page<CategoryInfoRespVo> results = this.baseMapper.findPage(page, vo);
        return JsonResult.success(new PageUtils(results), UtilMessage.GET_MESSAGE_SUCCESS);
    }

    @Override
    public CategoryInfoDetailRespVo detail(Long categoryInfoId) {
        CategoryInfoDetailRespVo detailVo = this.baseMapper.detail(categoryInfoId);
        if (ObjectUtil.isEmpty(detailVo)) {
            throw new BusinessException(UtilMessage.CATEGORY_NOEXIST);
        }
        //查询资质图片
        LambdaQueryWrapper<HsayiCategoryImg> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(HsayiCategoryImg::getCat_id, categoryInfoId);
        List<HsayiCategoryImg> imgs = categoryImgService.list(wrapper);
        if (!imgs.isEmpty()) {
            Map<String, List<HsayiCategoryImg>> imgMap = imgs.stream().collect(Collectors.groupingBy(HsayiCategoryImg::getType));
            List<HsayiCategoryImg> list = imgMap.get(UtilConst.license);
            if (list == null) {
                detailVo.setLicenseImgs(Lists.newArrayList());
            } else {
                detailVo.setLicenseImgs(list.stream().map(HsayiCategoryImg::getUrl).collect(Collectors.toList()));
            }
            List<HsayiCategoryImg> list2 = imgMap.get(UtilConst.certificate);
            if (list2 == null) {
                detailVo.setCertificateImgs(Lists.newArrayList());
            } else {
                detailVo.setCertificateImgs(list2.stream().map(HsayiCategoryImg::getUrl).collect(Collectors.toList()));
            }
        }
        return detailVo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public JsonResult add(CategoryInfoAddRepVo vo) {
        //验证类目唯一
        LambdaQueryWrapper<HsayiCategoryInfo> cwrapper = new LambdaQueryWrapper<>();
        cwrapper.eq(HsayiCategoryInfo::getLevel1, vo.getLevel1());
        cwrapper.eq(HsayiCategoryInfo::getLevel2, vo.getLevel2());
        cwrapper.eq(HsayiCategoryInfo::getLevel3, vo.getLevel3());
        List<HsayiCategoryInfo> list = this.list(cwrapper);
        if (!list.isEmpty()) {
            throw new BusinessException(UtilMessage.CATEGORY_EXIST);
        }
        cwrapper = new LambdaQueryWrapper<>();
        cwrapper.eq(HsayiCategoryInfo::getLevel3_name, vo.getLevel3_name());
        List<HsayiCategoryInfo> list2 = this.list(cwrapper);
        if (!list2.isEmpty()) {
            throw new BusinessException(UtilMessage.CATEGORY_NAME_EXIST);
        }
        HsayiCategoryInfo newEntity = new HsayiCategoryInfo();
        BeanUtil.copyProperties(vo, newEntity);
        //提交微信审核获取审核id
        String auditId = wechatFacade.auditCategory(vo);
        newEntity.setAudit_id(auditId);
        this.save(newEntity);
        //插入图片
        List<HsayiCategoryImg> categoryImgs = Lists.newArrayList();
        if (ObjectUtil.isNotEmpty(vo.getLicenseImgs())) {
            String[] arr = vo.getLicenseImgs().split(",");
            for (String str : arr) {
                HsayiCategoryImg ceImg = new HsayiCategoryImg();
                ceImg.setType(UtilConst.license);
                ceImg.setCat_id(newEntity.getCategory_info_id());
                ceImg.setUrl(str);
                categoryImgs.add(ceImg);
            }
            String[] arr2 = vo.getCertificateImgs().split(",");
            for (String str : arr2) {
                HsayiCategoryImg ceImg = new HsayiCategoryImg();
                ceImg.setType(UtilConst.certificate);
                ceImg.setCat_id(newEntity.getCategory_info_id());
                ceImg.setUrl(str);
                categoryImgs.add(ceImg);
            }
            categoryImgService.saveBatch(categoryImgs);
        }
        return JsonResult.success(UtilMessage.CATEGORY_SUBMIT_MESSAGE_SUCCESS);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public JsonResult edit(CategoryInfoEditRepVo vo) {
        HsayiCategoryInfo oldEntity = this.getById(vo.getCategory_info_id());
        if (ObjectUtil.isEmpty(oldEntity)) {
            throw new BusinessException(UtilMessage.CATEGORY_NOEXIST);
        }
        //验证类目唯一
        LambdaQueryWrapper<HsayiCategoryInfo> cwrapper = new LambdaQueryWrapper<>();
        cwrapper.eq(HsayiCategoryInfo::getLevel1, vo.getLevel1());
        cwrapper.eq(HsayiCategoryInfo::getLevel2, vo.getLevel2());
        cwrapper.eq(HsayiCategoryInfo::getLevel3, vo.getLevel3());
        cwrapper.ne(HsayiCategoryInfo::getCategory_info_id, vo.getCategory_info_id());
        List<HsayiCategoryInfo> list = this.list(cwrapper);
        if (!list.isEmpty()) {
            throw new BusinessException(UtilMessage.CATEGORY_EXIST);
        }
        cwrapper = new LambdaQueryWrapper<>();
        cwrapper.eq(HsayiCategoryInfo::getLevel3_name, vo.getLevel3_name());
        cwrapper.ne(HsayiCategoryInfo::getCategory_info_id, vo.getCategory_info_id());
        List<HsayiCategoryInfo> list2 = this.list(cwrapper);
        if (!list2.isEmpty()) {
            throw new BusinessException(UtilMessage.CATEGORY_NAME_EXIST);
        }
        HsayiCategoryInfo newEntity = new HsayiCategoryInfo();
        BeanUtil.copyProperties(vo, newEntity);
        BeanUtil.copyProperties(newEntity, oldEntity);
        //提交微信审核获取审核id
        String auditId = wechatFacade.auditCategory(vo);
        oldEntity.setAudit_id(auditId);
        oldEntity.setReject_reason("");
        oldEntity.setStatus("0");
        oldEntity.setCreate_time(new Date());
        this.updateById(oldEntity);
        //全量删除
        categoryImgService.remove(new LambdaQueryWrapper<HsayiCategoryImg>().eq(HsayiCategoryImg::getCat_id, oldEntity.getCategory_info_id()));
        //插入图片
        List<HsayiCategoryImg> categoryImgs = Lists.newArrayList();
        if (ObjectUtil.isNotEmpty(vo.getLicenseImgs())) {
            String[] arr = vo.getLicenseImgs().split(",");
            for (String str : arr) {
                HsayiCategoryImg ceImg = new HsayiCategoryImg();
                ceImg.setType(UtilConst.license);
                ceImg.setCat_id(newEntity.getCategory_info_id());
                ceImg.setUrl(str);
                categoryImgs.add(ceImg);
            }
            String[] arr2 = vo.getCertificateImgs().split(",");
            for (String str : arr2) {
                HsayiCategoryImg ceImg = new HsayiCategoryImg();
                ceImg.setType(UtilConst.certificate);
                ceImg.setCat_id(newEntity.getCategory_info_id());
                ceImg.setUrl(str);
                categoryImgs.add(ceImg);
            }
            categoryImgService.saveBatch(categoryImgs);
        }
        return JsonResult.success(UtilMessage.CATEGORY_EDIT_MESSAGE_SUCCESS);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public JsonResult deleteByIds(Long[] categoryInfoIds) {
        categoryImgService.remove(new LambdaQueryWrapper<HsayiCategoryImg>().in(HsayiCategoryImg::getCat_id, categoryInfoIds));
        this.baseMapper.delete(new LambdaQueryWrapper<HsayiCategoryInfo>().in(HsayiCategoryInfo::getCategory_info_id, categoryInfoIds));
        return JsonResult.success(UtilMessage.DEL_MESSAGE_SUCCESS);
    }

    @Override
    public JsonResult authedEdit(CategoryInfoAuthedEditRepVo vo) {
        HsayiCategoryInfo oldEntity = this.getById(vo.getCategory_info_id());
        if (ObjectUtil.isEmpty(oldEntity)) {
            throw new BusinessException(UtilMessage.CATEGORY_NOEXIST);
        }
        LambdaQueryWrapper<HsayiCategoryInfo> cwrapper = new LambdaQueryWrapper<>();
        cwrapper.eq(HsayiCategoryInfo::getLevel3_name, vo.getLevel3_name());
        cwrapper.ne(HsayiCategoryInfo::getCategory_info_id, vo.getCategory_info_id());
        List<HsayiCategoryInfo> list2 = this.list(cwrapper);
        if (!list2.isEmpty()) {
            throw new BusinessException(UtilMessage.CATEGORY_NAME_EXIST);
        }
        HsayiCategoryInfo newEntity = new HsayiCategoryInfo();
        BeanUtil.copyProperties(vo, newEntity);
        BeanUtil.copyProperties(newEntity, oldEntity);
        this.updateById(oldEntity);
        return JsonResult.success(UtilMessage.UPDATE_MESSAGE_SUCCESS);
    }
}
