package com.ads.common.aspect;

import com.ads.common.annotation.TokenValidator;
import com.ads.common.handler.WeChatHandler;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.stereotype.Component;

import java.lang.reflect.Method;

/**
 * [微信token切面]
 *
 * @author huzhihao
 * @date 2021/12/22 14:37
 */
@Slf4j
@Aspect
@Component
public class WechatTokenAspect {

    /**
     * [配置织入点]
     *
     * @param
     * @return void
     * @author huzhihao
     * @date 2021/12/21 14:12
     */
    @Pointcut("@annotation(com.ads.common.annotation.TokenValidator)")
    public void TokenValidatorCut() {
    }

    /**
     * [方法执行前]
     *
     * @param point
     * @return void
     * @author huzhihao
     * @date 2021/12/21 14:12
     */
    @Before("TokenValidatorCut()")
    public void doBefore(JoinPoint point) throws Throwable {
        log.info("===>方法执行前校验access_token");
        getAccessToken(point);
    }

    /**
     * [校验token有效性并判断是否更新]
     *
     * @param joinPoint
     * @return void
     * @author huzhihao
     * @date 2021/12/21 14:11
     */
    protected void getAccessToken(final JoinPoint joinPoint) {
        TokenValidator annotationLog = getAnnotationLog(joinPoint);
        if (annotationLog == null) {
            return;
        }
        WeChatHandler.accessToken();
    }

    /**
     * [是否存在注解，如果存在就获取]
     *
     * @param joinPoint
     * @return com.example.wechat_event_demo.annotation.TokenValidator
     * @author huzhihao
     * @date 2021/12/21 14:11
     */
    private TokenValidator getAnnotationLog(JoinPoint joinPoint) {
        Signature signature = joinPoint.getSignature();
        MethodSignature methodSignature = (MethodSignature) signature;
        Method method = methodSignature.getMethod();

        if (method != null) {
            return method.getAnnotation(TokenValidator.class);
        }
        return null;
    }
}
