package com.ads.common.interceptor;

import cn.hutool.core.util.StrUtil;
import com.ads.business.service.data.GetLoginOutData;
import com.ads.common.constant.UtilConst;
import com.ads.common.exception.BusinessException;
import com.ads.common.exception.LoginException;
import com.ads.common.redis.RedisKeyConstant;
import com.ads.common.redis.RedisManager;
import com.ads.common.util.UtilMessage;
import com.alibaba.fastjson.JSONObject;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;

/**
 * @author yuqiang
 * @Description 权限拦截
 * @Date 2020/3/18 15:33
 */
@Component
public class SecurityInterceptor implements HandlerInterceptor {


    @Resource
    private RedisManager redisManager;

    private static final List<String> allowUrls = new ArrayList<>();

    static {
        allowUrls.add("/login");
        allowUrls.add("/logout");
    }

    /**
     * 在调用controller具体方法前拦截
     */
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object object) throws Exception {
        String path = request.getServletPath();
        if (path.indexOf("swagger") != -1 || path.indexOf("acutator") != -1 || path.indexOf("scheduler") != -1) {
            return true;
        }
        if (path.contains(UtilConst.URL_ERROR)) {
            System.err.println("错误信息是:" + request.getServletPath() + "错误方法:" + request.getMethod());
            throw new BusinessException(UtilMessage.HTTP_ERROR);
        }
        if (allowUrls.contains(path)) {
            return true;
        }
        String method = request.getMethod();
        if (UtilConst.REQUEST_METHOD_OPTIONS.equalsIgnoreCase(method)) {
            return true;
        }
        String token = request.getHeader(UtilConst.LOGIN_TOKEN);
        if (StrUtil.isBlank(token)) {
            throw new LoginException();
        } else {
            String json =(String)redisManager.get(RedisKeyConstant.PREFIX + token);
            GetLoginOutData userInfo =JSONObject.parseObject(json,GetLoginOutData.class);
            if (userInfo == null) {
                throw new LoginException();
            }
        }
        return true;
    }

    /**
     * 完成页面的render后调用
     */
    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object object, Exception exception) throws Exception {
    }

    /**
     * 在调用controller具体方法后拦截
     */
    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object object, ModelAndView modelAndView) throws Exception {
    }
}
