package com.ads.common.util;

import com.ads.business.service.data.GetLoginOutData;
import com.ads.common.exception.LoginException;
import com.ads.common.redis.RedisKeyConstant;
import com.ads.common.redis.RedisManager;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.PostConstruct;
import javax.servlet.http.HttpServletRequest;

/**
 * 获取用户信息
 *
 * @author: stamm
 * @Date: 2021/06/09/15:48
 */
@Component
@Slf4j
public class ContextUtils {


    @Autowired
    private RedisManager redisManager2;

    private static RedisManager redisManager;

    @PostConstruct
    public void beforeInit() {
        redisManager = redisManager2;
    }

    public static HttpServletRequest getRequest() {
        return ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
    }

    public static String getHeader(String key) {
        return getRequest().getHeader(key);
    }

    /**
     * 获取用户信息
     *
     * @return
     */
    public static GetLoginOutData getUserInfo() {

        GetLoginOutData userInfo = null;
        //获取token
        String token = getHeader(UtilConst.LOGIN_TOKEN);
        //String token = null;
        //验证
        if (StringUtils.isNotBlank(token)) {
            String json = (String) redisManager.get(RedisKeyConstant.PREFIX + token);
            userInfo = JSONObject.parseObject(json, GetLoginOutData.class);
            if (userInfo == null) {
                throw new LoginException();
            }
        } else {
            userInfo = new GetLoginOutData();
            userInfo.setUserId(1L);
            userInfo.setName("管理员");
        }
        return userInfo;
    }

    /**
     * 获取用户id
     *
     * @return
     */
    public static Long getUserId() {
        Long userId = 1L;
        //获取token
        String token = getHeader(UtilConst.LOGIN_TOKEN);
        //String token = null;
        //验证
        if (StringUtils.isNotBlank(token)) {
            String json = (String) redisManager.get(RedisKeyConstant.PREFIX + token);
            GetLoginOutData userInfo = JSONObject.parseObject(json, GetLoginOutData.class);
            if (userInfo == null) {
                throw new LoginException();
            } else {
                userId = userInfo.getUserId();
            }
        }
        return userId;
    }
}
